%%% Econ 203C (Spring 2009)
%%% Problem Set II, Question 2
%%% Monte Carlo Simulation of Eicker White robust covariance estimator

clear all;              % clear all the variables
clc;                    % clear the screen

% Set primitive variables
R=1000;                 % number of Monte Carlo simulation
T=100;                  % sample size in each simulation
k=5;                    % dimension of parameter beta
sigma2=1;               % true parameter for variances of U

% Intermediate parameters
beta=[1:k]';            % true parameter
Ex=zeros(1,k);          % Ex = 0
VARx=eye(k);            % Var(x) = identity

% Calculate True Omega
Exx=Ex*Ex'+VARx;        % Exx' = Ex Ex' + Var(x)
Omega=sigma2*Exx^(-1);  % under conditional homoskedasticity assumption

% Initialize the sequence of random numbers
randn('seed',2032009);

% Initialize the estimates matrix
beta_est = [];
Omega1_bias = [];
Omega2_bias = [];

% Start iteration
for r=1:R
    % Data Generating Process
    x = ones(T,1) * Ex + randn(T,k) * chol(VARx);
    y = x * beta + sqrt(sigma2) * randn(T,1);

    % Least Squares Estimator
    beta_hat = inv(x'*x)*x'*y;
    u_hat = y - x * beta_hat;
    Omega1_hat = u_hat' * u_hat / T * inv(x'*x/T);
                        % Estimate asymptotic variance under the assumption
                        % = sigma2 * Exx'^(-1) .=. u_hat'u_hat * (x'x)^(-1)
    uxx_hat = (u_hat * ones(1,k) .* x)' * (u_hat * ones(1,k) .* x) / T;
                        % Eu^2xx' = E[ (ux) (ux)' ]
    Omega2_hat = inv(x'*x/T) * uxx_hat * inv(x'*x/T);
                        % Estimate asymptotic variance without the assumption
                        % = Exx'^(-1) * Eu^2xx' * Exx'(-1)
    
    % Stack estimates for later use
    beta_est = [beta_est;beta_hat'];
    Omega1_bias(:,:,r) = Omega1_hat - Omega;
    Omega2_bias(:,:,r) = Omega2_hat - Omega;
end

% Initialize printing format
PF1 = [];
PF2 = [];
for i=1:k
    PF1 = [PF1 '----------'];
    PF2 = [PF2 '%10.4f'];
end

% Print the result
fprintf('\n');
fprintf('*****  Monte Carlo Simulation Result  *****\n');
fprintf('%5d times simulation of sample size %5d\n',R,T);
fprintf('===========================================\n');
fprintf('\n');

fprintf(['  True  beta  ' PF2 '\n'],beta);
fprintf(['--------------' PF1 '\n']);
fprintf(['              (estimates of beta)\n']);
fprintf(['     Mean     ' PF2 '\n'],mean(beta_est));
fprintf([' Standard Dev ' PF2 '\n'],std(beta_est));
fprintf(['--------------' PF1 '\n']);
fprintf(['  True Omega\n']);
fprintf(['              ' PF2 '\n'],Omega');
fprintf(['--------------' PF1 '\n']);
fprintf(['  Mean  Bias  (estimates of Omega1: correct formula)\n']);
fprintf(['              ' PF2 '\n'],mean(Omega1_bias,3)');
fprintf(['              (estimates of Omega2: White formula)\n']);
fprintf(['              ' PF2 '\n'],mean(Omega2_bias,3)');
fprintf(['--------------' PF1 '\n']);
fprintf([' Standard Dev (estimates of Omega1: correct formula)\n']);
fprintf(['              ' PF2 '\n'],std(Omega1_bias,0,3)');
fprintf(['              (estimates of Omega2: White formula)\n']);
fprintf(['              ' PF2 '\n'],std(Omega2_bias,0,3)');
fprintf(['--------------' PF1 '\n']);
fprintf([' R M S Error  (estimates of Omega1: correct formula)\n']);
fprintf(['              ' PF2 '\n'],sqrt(mean(Omega1_bias.^2,3))');
fprintf(['              (estimates of Omega2: White formula)\n']);
fprintf(['              ' PF2 '\n'],sqrt(mean(Omega2_bias.^2,3))');
fprintf(['--------------' PF1 '\n']);
fprintf(['\n']);

% Norm Statistics
Norm_Omega1 = sqrt(sum(sum(Omega1_bias.^2,1),2));
Norm_Omega2 = sqrt(sum(sum(Omega2_bias.^2,1),2));
fprintf(['  Norm Stats      Omega1    Omega2\n']);
fprintf(['----------------------------------\n']);
fprintf(['     Mean     %10.4f%10.4f\n'],mean(Norm_Omega1),mean(Norm_Omega2));
fprintf([' Standard Dev %10.4f%10.4f\n'],std(Norm_Omega1),std(Norm_Omega2));
fprintf(['    Minimum   %10.4f%10.4f\n'],min(Norm_Omega1),min(Norm_Omega2));
fprintf(['    Median    %10.4f%10.4f\n'],median(Norm_Omega1),median(Norm_Omega2));
fprintf(['    Maximum   %10.4f%10.4f\n'],max(Norm_Omega1),max(Norm_Omega2));
